{% extends "dashboard_app/base.html" %}
{% load i18n %}
{% load humanize %}
{% load pagination_tags %}


{% block title %}
{{ block.super }} | {% trans "Test Runs" %} | {{ test_run }} | {% trans "Hardware Context" %}
{% endblock %}


{% block breadcrumbs %}
<li><a href="{% url dashboard_app.views.bundle_stream_list %}">{% trans "Bundle Streams" %}</a></li>
<li><a href="{{ test_run.bundle.bundle_stream.get_absolute_url }}">{{ test_run.bundle.bundle_stream }}</a></li>
<li><a href="{{ test_run.get_absolute_url }}">{{ test_run }}</a></li>
<li><a href="{% url dashboard_app.views.test_run_hardware_context test_run.analyzer_assigned_uuid %}">{% trans "Hardware Context" %}</a></li>
{% endblock %}


{% block sidebar %}
{% endblock %}


{% block content %}
<h2>Hardware Devices</h2>
<dl>
{% autopaginate test_run.devices.all as hardware_devices %}
{% for hardware_device in hardware_devices %}
<dt>{{ hardware_device.description }}</dt>
<dd>
<table>
  {% for attribute in hardware_device.attributes.all %}
  <tr>
    <th>{{ attribute.name }}</th>
    <td>{{ attribute.value }}</td>
  </tr>
  {% endfor %}
</table>
</dd>
{% empty %}
<em>There are no hardware devices associated with this test run</em>
{% endfor %}
</dl>
{% paginate %}
{% endblock %}
